// @ts-ignore
import { Util } from 'bootstrap';

const NAME = 'layout';
const DATA_KEY = 'ca.bs.layout';
const EVENT_KEY = `.${DATA_KEY}`;
const DATA_API_KEY = '.data-api';
const JQUERY_NO_CONFLICT = $.fn[NAME];
const TRANSITION_DURATION = 250;

const ClassNames = {
    SHOW: 'show',
    COLLAPSE: 'collapse',
    COLLAPSING: 'collapsing',
    COLLAPSED: 'collapsed',
};

const Default: LayoutConfig = {
    root: '.ca-root',
    mode: 'toggle',
    durationCssElement: '.ca-sidebar',
};

const Event = {
    SHOW: `show${EVENT_KEY}`,
    SHOWN: `shown${EVENT_KEY}`,
    HIDE: `hide${EVENT_KEY}`,
    HIDDEN: `hidden${EVENT_KEY}`,
    CLICK_DATA_API: `click${EVENT_KEY}${DATA_API_KEY}`,
};

const Selector = {
    DATA_LAYOUT: '[data-layout]',
};

interface LayoutConfig {
    layout?: string;
    root?: string;
    mode?: string;
    durationCssElement?: string;
}

export default class Layout {
    private readonly _$element: JQuery;
    private readonly _$root: JQuery;
    private readonly _config: LayoutConfig;

    private _resolvedDuration: number | undefined;

    get elementType(): string {
        return this._config.layout || '';
    }

    get mode(): string {
        return this._config.mode || '';
    }

    constructor(element: HTMLElement, config: LayoutConfig) {
        this._$element = $(element);
        this._$root = $(config.root);
        this._config = config;
    }

    toggle() {
        if (this._$root.hasClass(`${this.elementType}-${ClassNames.SHOW}`)) {
            this.hide();
        } else {
            this.show();
        }
    }

    show() {
        const collapse = `${this.elementType}-${ClassNames.COLLAPSE}`;
        const collapsing = `${this.elementType}-${ClassNames.COLLAPSING}`;
        const show = `${this.elementType}-${ClassNames.SHOW}`;

        if (this._getTransitioning() || this._$root.hasClass(show)) {
            return;
        }
        const startEvent = $.Event(Event.SHOW, { layout: this.elementType });
        this._$root.trigger(startEvent);
        if (startEvent.isDefaultPrevented()) {
            return;
        }
        this._$root
            .removeClass(collapse)
            .addClass(collapsing)
            .addClass(show);

        this._setTransitioning(true);
        const complete = () => {
            this._$root.removeClass(collapsing).addClass(collapse);
            this._setTransitioning(false);
            this._$root.trigger(Event.SHOWN, { layout: this.elementType });
        };
        if (!Util.supportsTransitionEnd()) {
            complete();
            return;
        }
        this._$root.one(Util.TRANSITION_END, complete).emulateTransitionEnd(this._getCalculatedTransitionDuration());
    }

    hide() {
        const collapse = `${this.elementType}-${ClassNames.COLLAPSE}`;
        const collapsing = `${this.elementType}-${ClassNames.COLLAPSING}`;
        const show = `${this.elementType}-${ClassNames.SHOW}`;

        if (this._getTransitioning() || !this._$root.hasClass(show)) {
            return;
        }

        const startEvent = $.Event(Event.HIDE, { layout: this.elementType });
        this._$root.trigger(startEvent);
        if (startEvent.isDefaultPrevented()) {
            return;
        }
        this._$root
            .addClass(collapsing)
            .removeClass(collapse)
            .removeClass(show);
        this._setTransitioning(true);
        const complete = () => {
            this._setTransitioning(false);
            this._$root
                .removeClass(collapsing)
                .addClass(collapse)
                .trigger(Event.HIDDEN, { layout: this.elementType });
        };
        (this._$root.one(Util.TRANSITION_END, complete) as any).emulateTransitionEnd(this._getCalculatedTransitionDuration());
    }

    private _getTransitioning() {
        let transitioning = this._$root.data('ca.transitioning');
        if (typeof transitioning === 'undefined') {
            transitioning = {};
        }
        if (typeof transitioning[this.elementType] === 'undefined') {
            transitioning[this.elementType] = false;
        }
        this._$root.data('ca.transitioning', transitioning);
        return transitioning[this.elementType];
    }

    private _setTransitioning(val: boolean) {
        let transitioning = this._$root.data('ca.transitioning');
        if (typeof transitioning === 'undefined') {
            transitioning = {};
        }
        transitioning[this.elementType] = val;
        this._$root.data('ca.transitioning', transitioning);
    }

    private _getElementTypeKey() {
        const match = /\[data-(\w+)\]/gi.exec(Selector.DATA_LAYOUT);
        return match ? match[1] : '';
    }

    private _getCalculatedTransitionDuration(): number {
        if (this._resolvedDuration) return this._resolvedDuration;

        if (this._config.durationCssElement) {
            const target = document.querySelector(this._config.durationCssElement);
            if (target) {
                const computed = getComputedStyle(target);
                const duration = computed['transitionDuration'];
                if (duration) {
                    const isMillisecond = duration.indexOf('ms') >= 0;
                    const match = duration.match(/\d+(\.\d+)?/g);
                    if (match) {
                        const numbers = parseFloat(match[0]);
                        this._resolvedDuration = Math.ceil(isMillisecond ? numbers : numbers * 1000);
                        return this._resolvedDuration;
                    }
                }
            }
        }
        this._resolvedDuration = TRANSITION_DURATION;
        return this._resolvedDuration;
    }

    // tslint:disable-next-line:naming-convention
    static _jQueryInterface(this: HTMLElement, config: LayoutConfig | string) {
        const $this = $(this);
        let method = config;
        let data = $this.data(DATA_KEY) as Layout;
        if (!data) {
            const cfg = {
                ...Default,
                ...(typeof config === 'object' && config),
            };
            cfg.layout = $this.data('layout');
            data = new Layout(this, cfg);
            $this.data(DATA_KEY, data);
            method = data.mode;
        }
        if (typeof method === 'string') {
            if (typeof data[method] === 'undefined') {
                throw new Error(`No method named "${method}"`);
            }
            data[method]();
        }
    }
}

$(document).on(Event.CLICK_DATA_API, Selector.DATA_LAYOUT, event => {
    if (event.currentTarget.tagName === 'A') {
        event.preventDefault();
    }
    const $trigger = $(event.currentTarget);
    const data = $trigger.data(DATA_KEY) as Layout;
    const config = data ? data.mode : $trigger.data();
    Layout._jQueryInterface.call($trigger[0], config);
});

$.fn[NAME] = Layout._jQueryInterface;
$.fn[NAME].Constructor = Layout;
$.fn[NAME].noConflict = function() {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Layout._jQueryInterface;
};
