const merge = require('webpack-merge');
const path = require('path');
const webpack = require('webpack');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');

const bundleOutputDir = './dist';

module.exports = (env) => {
    const isDevBuild = !(env && env.prod);
    const isHost = env && env.host;

    const pages = [
        {
            src: 'main',
            input: 'index.html',
            output: 'index.html',
        },
    ];

    let config = {
        stats: { modules: false },
        context: __dirname,
        resolve: { extensions: ['.js', '.ts'] },
        entry: {
            main: './src/scripts/boot.ts',
            'site.css': './src/styles/site.scss',
            'index.html': './src/views/index.html',
        },
        module: {
            rules: [
                {
                    test: /\.ts$/,
                    include: path.join(__dirname, 'src', 'scripts'),
                    use: [
                        {
                            loader: 'ts-loader',
                            options: {
                                transpileOnly: true,
                            },
                        },
                    ],
                },
                {
                    test: /\.css$/,
                    use: isDevBuild
                        ? ['style-loader', 'css-loader']
                        : ExtractTextPlugin.extract({
                            use: 'css-loader?minimize',
                        }),
                },
                {
                    test: /\.scss$/,
                    use: [
                        {
                            loader: 'style-loader',
                        },
                        {
                            loader: 'css-loader',
                        },
                        {
                            loader: 'sass-loader',
                        },
                    ],
                },
                {
                    test: /\.(png|jpg|jpeg|gif|svg)$/,
                    use: [
                        {
                            loader: 'url-loader',
                            options: {
                                limit: 25000,
                            },
                        },
                    ],
                },
                {
                    test: /\.html$/,
                    use: 'html-loader',
                },
            ],
        },
        output: {
            path: path.join(__dirname, bundleOutputDir),
            filename: '[name].js',
            publicPath: '/dist/',
        },
        plugins: [
            new webpack.DefinePlugin({
                'process.env': {
                    NODE_ENV: JSON.stringify(isDevBuild ? 'development' : 'production'),
                },
            }),
            new webpack.DllReferencePlugin({
                context: __dirname,
                // eslint-disable-next-line global-require
                manifest: require('./assets/vendor-manifest.json'),
            }),
        ],
    };

    if (isDevBuild) {
        // Plugins that apply in development builds only
        const plugins = [
            new webpack.SourceMapDevToolPlugin({
                filename: '[file].map', // Remove this line if you prefer inline source maps
                // moduleFilenameTemplate: path.relative('/', '[resourcePath]'), // Point source map entries to the original file locations on disk
            }),
            new webpack.NamedModulesPlugin(),
        ];

        config = merge(config, {
            plugins,
        });
    }

    if (isHost) {
        const plugins = pages
            .map(el =>
                new HtmlWebpackPlugin({
                    inject: false,
                    chunks: [el.src, 'site.css', el.input],
                    headInject: ['site.css'],
                    bodyInject: [el.src],
                    filename: el.output,
                    template: 'src/template.js',
                    content: [el.input],
                }))
            .concat(new webpack.HotModuleReplacementPlugin());

        plugins.push(new webpack.ProvidePlugin({
            $: 'jquery',
            jQuery: 'jquery',
            'window.jQuery': 'jquery',
            Popper: ['popper.js', 'default'],
        }));

        config = merge(config, {
            devServer: {
                port: 9000,
                hot: true,
                publicPath: '/',
            },
            plugins,
        });
    }

    return [config];
};
